#
# spec file for package ghc-dejafu
#
# Copyright (c) 2017 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%global pkg_name dejafu
Name:           ghc-%{pkg_name}
Version:        0.4.0.0
Release:        8.9
Summary:        Systematic testing for Haskell concurrency
License:        MIT
Group:          Development/Libraries/Haskell
URL:            https://hackage.haskell.org/package/%{pkg_name}
Source0:        https://hackage.haskell.org/package/%{pkg_name}-%{version}/%{pkg_name}-%{version}.tar.gz
BuildRequires:  ghc-Cabal-devel
BuildRequires:  ghc-concurrency-devel
BuildRequires:  ghc-containers-devel
BuildRequires:  ghc-deepseq-devel
BuildRequires:  ghc-dpor-devel
BuildRequires:  ghc-exceptions-devel
BuildRequires:  ghc-monad-loops-devel
BuildRequires:  ghc-mtl-devel
BuildRequires:  ghc-ref-fd-devel
BuildRequires:  ghc-rpm-macros
BuildRequires:  ghc-semigroups-devel
BuildRequires:  ghc-transformers-base-devel
BuildRequires:  ghc-transformers-devel

%description
/[Déjà Fu is] A martial art in which the user's limbs move in time as well as
space, […] It is best described as "the feeling that you have been kicked in
the head this way before"/ -- Terry Pratchett, Thief of Time

Concurrency is nice, deadlocks and race conditions not so much. The 'Par' monad
family, as defined in
<https://hackage.haskell.org/package/abstract-par/docs/Control-Monad-Par-Class.html
abstract-par> provides deterministic parallelism, but sometimes we can tolerate
a bit of nondeterminism.

This package builds on the <https://hackage.haskell.org/package/concurrency
concurrency> package by enabling you to systematically and deterministically
test your concurrent programs.

== Déjà Fu with 'IO':

The core assumption underlying Déjà Fu is that any apparent nondeterminism
arises purely from the scheduling behaviour. To put it another way, a given
computation, parametrised with a fixed set of scheduling decisions, is
deterministic.

Whilst this assumption may not hold in general when 'IO' is involved, you
should strive to produce test cases where it does.

== Memory Model

The testing functionality supports a few different memory models, for
computations which use non-synchronised `CRef` operations. The supported models
are:

* __Sequential Consistency:__ A program behaves as a simple interleaving of the
actions in different threads. When a CRef is written to, that write is
immediately visible to all threads.

* __Total Store Order (TSO):__ Each thread has a write buffer. A thread sees
its writes immediately, but other threads will only see writes when they are
committed, which may happen later. Writes are committed in the same order that
they are created.

* __Partial Store Order (PSO):__ Each CRef has a write buffer. A thread sees
its writes immediately, but other threads will only see writes when they are
committed, which may happen later. Writes to different CRefs are not
necessarily committed in the same order that they are created.

If a testing function does not take the memory model as a parameter, it uses
TSO.

See the <https://github.com/barrucadu/dejafu README> for more details.

%package devel
Summary:        Haskell %{pkg_name} library development files
Group:          Development/Libraries/Haskell
Requires:       %{name} = %{version}-%{release}
Requires:       ghc-compiler = %{ghc_version}
Requires(post): ghc-compiler = %{ghc_version}
Requires(postun): ghc-compiler = %{ghc_version}

%description devel
This package provides the Haskell %{pkg_name} library development files.

%prep
%setup -q -n %{pkg_name}-%{version}

%build
%ghc_lib_build

%install
%ghc_lib_install

%post devel
%ghc_pkg_recache

%postun devel
%ghc_pkg_recache

%files -f %{name}.files
%doc LICENSE

%files devel -f %{name}-devel.files

%changelog
* Thu Aug  3 2017 psimons@suse.com
- Updated with latest spec-cleaner version 0.9.8-8-geadfbbf.
* Thu Sep 15 2016 psimons@suse.com
- Update to version 0.4.0.0 revision 0 with cabal2obs.
* Mon Jul 25 2016 psimons@suse.com
- Update to version 0.3.2.1 revision 0 with cabal2obs.
* Sun Jul 10 2016 psimons@suse.com
- Update to version 0.3.2.0 revision 0 with cabal2obs.
